/*
 * Copyright 2015 Riccardo Padovani <riccardo@rpadovani.com>
 *
 * This file is part of falldown.
 *
 * falldown is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * falldown is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

import QtQml 2.2
import QtQuick 2.3
import QtSensors 5.0
import QtSystemInfo 5.0
import QtMultimedia 5.4
import Ubuntu.Components 1.2
import Bacon2D 1.0

import "components"
import "scenes"
import "../js/game.js" as Game

MainView {
    id: mainView
    applicationName: "falldown.rpadovani"

    height: units.gu(68)
    width: units.gu(44)

    property var version: "0.3.0"
    property real velocity: units.gu(0.3)

    property var soundPrefix: "themes/default/sounds/" //+ settings.theme + "/sounds/"
    property var imgPrefix: "themes/" + settings.theme + "/img/"

    Game {
        id: game
        anchors.centerIn: parent
        anchors.fill: parent

        currentScene: mainScene

        ScreenSaver { screenSaverEnabled: !gameScene.running }

        Settings {
            id: settings
            property int highScore: 0
            property bool mute: false
            property var control: "tilt"
            property real tiltSensivity: 1.0
            property real touchSensivity: 1.0
            property var theme: "default"

            onMuteChanged: {
                if (mute) {
                    soundtrack.stop();
                } else {
                    soundtrack.play();
                }
            }
        }

        Connections {
            target: Qt.application
            onActiveChanged: {
                if (Qt.application.active && !settings.mute) {
                    soundtrack.play();
                } else {
                    soundtrack.pause();
                }
            }
        }

        MainScene {
            id: mainScene
            anchors.fill: parent
        }

        GameScene {
            id: gameScene
            anchors.fill: parent

            gravity: Qt.point(0, 20)
        }

        AboutScene {
            id: aboutScene
            anchors.fill: parent
        }

        EndScene {
            id: endScene
            anchors.fill: parent
        }

        SettingsScene {
            id: settingsScene
            anchors.fill: parent
        }

        Floor { id: floor }
        Hole { id: hole }
        Ball { id: ball }
        PowerUp { id: powerUp }

        Timer {
            id: smallerBallTimer
            interval: 5000

            onTriggered: gameScene.smallerBall = false;
        }

        Timer {
            id: slowTimeTimer
            interval: 2125

            onTriggered: gameScene.slowTime = false;
        }

        Timer {
            id: balloonTimer
            interval: 3000

            onTriggered: gameScene.balloon = false;
        }

        Timer {
            id: glueTimer
            interval: 8000

            onTriggered: gameScene.glue = false;
        }

        Timer {
            id: wineTimer
            interval: 5000

            onTriggered: gameScene.wine = false;
        }

        Timer {
            id: newLifeTimer
            interval: 1500

            onTriggered: {
                velocity = gameScene.oldVelocity;
                Game.addBall();
            }
        }

        SoundEffect {
            id: balloonSound
            muted: settings.mute
            loops: 0
            source: Qt.resolvedUrl(mainView.soundPrefix + "Balloon.wav")
        }

        SoundEffect {
            id: clockSound
            muted: settings.mute
            loops: 0
            source: Qt.resolvedUrl(mainView.soundPrefix + "Clock.wav")
        }

        SoundEffect {
            id: glueSound
            muted: settings.mute
            loops: 0
            source: Qt.resolvedUrl(mainView.soundPrefix + "Glue.wav")
        }

        SoundEffect {
            id: heartSound
            muted: settings.mute
            loops: 0
            source: Qt.resolvedUrl(mainView.soundPrefix + "Heart.wav")
        }

        SoundEffect {
            id: multiplySound
            muted: settings.mute
            loops: 0
            source: Qt.resolvedUrl(mainView.soundPrefix + "Multiply.wav")
        }

        SoundEffect {
            id: shrinkSound
            muted: settings.mute
            loops: 0
            source: Qt.resolvedUrl(mainView.soundPrefix + "Shrink.wav")
        }

        SoundEffect {
            id: swollowSound
            muted: settings.mute
            loops: 0
            source: Qt.resolvedUrl(mainView.soundPrefix + "Swollow.wav")
        }

        SoundEffect {
            id: wineSound
            muted: settings.mute
            loops: 0
            source: Qt.resolvedUrl(mainView.soundPrefix + "Wine.wav")
        }

        Audio {
            id: soundtrack
            source: "file://" + soundsDir + "/soundtrack.mp3"
            muted: settings.mute
            loops: Audio.Infinite
            //autoPlay: true
            volume: 0.7
        }

        function changeGravity(direction) {
            var inverted = gameScene.wine ? -1 : 1;
            var ballVelocity = gameScene.glue ? 3 : 20;
            ballVelocity = ballVelocity * inverted * settings.touchSensivity;

            if (direction === "left") {
                gameScene.gravity = Qt.point(-ballVelocity, 20);
            } else if (direction === "right") {
                gameScene.gravity = Qt.point(ballVelocity, 20);
            } else if (direction === "reset") {
                gameScene.gravity = Qt.point(0, 20);
            } else {
                console.log("Wrong param for changeGravity()");
            }
        }

        Keys.onPressed: {
            if (event.key === Qt.Key_Left) {
                changeGravity("left");
            }

            if (event.key === Qt.Key_Right) {
                changeGravity("right");
            }
        }

        Keys.onReleased: changeGravity("reset");

        Accelerometer {
            active: settings.control == 'tilt'

            onReadingChanged: {
                var inverted = gameScene.wine ? -1 : 1;
                var ballVelocity = gameScene.glue ? -2 : -12;
                gameScene.gravity = Qt.point(settings.tiltSensivity * ballVelocity * reading.x * inverted, 20)
            }
        }
    }
}
